<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImportBase.php");	
	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkStockManager.php");	
	require_once("PolarisLinkImport_warehouse.php");	
		
	/*
	 * ex. : <stock sku="11" ean="22" warehouse="1" [w_price="5.00"] [s_price="10.00"]>10</stock>
	 */
	class PolarisLinkImport_stock extends PolarisLinkImportBase 
	{
		private $stockManager = null;		
		private $stock_reason = null;
		private $warehouses = null;
		private $storesMap = null;
		
		protected function OnInit()
		{
			$this->stockManager = StockManagerFactory::getManager();			
			$this->stock_reason = Configuration::get('PS_STOCK_MVT_INC_REASON_DEFAULT');
			$this->warehouses = $this->JobManager->getImporter("warehouse");

			$this->storesMap = json_decode(Configuration::get("POLARISLINK_STORESMAP"), true);
		}
		
		protected function OnImportData($data)
		{
			if (!$this->Link->OPT_ALLOW_I_STOCKS && !$this->Link->OPT_ALLOW_I_SALE_PRICES)
				return;	// Rien à faire ici dans ce cas là !
			
			// Ok, nous cherchons donc le produit prestashop correspondant
			// à la référence donnée
			$product = $this->GetProductCombinationFor((string) $data["sku"], 
								$this->Link->OPT_USE_EAN_STOCK ? (string) $data["ean"] : null, false, true);

			if ($product == NULL) // produit non trouvé 
				return;
				
			// Chargement du produit
			$c_product = $this->GetProduct($product["id_product"]);
			if (empty($c_product->id))
				return;	// Supprimé entre-temps ?

			// Autodétection d'un produit Polaris désactivé !
			if ($c_product->id_supplier == $this->Link->SupplierID &&
				$c_product->available_for_order == 0 &&
				$c_product->visibility == "none")
				return;	// Produit désactivé, on ne fait rien !

			$cmb_product = null;
			if (!empty($product["id_product_attribute"]))
			{
				$cmb_product = new Combination($product["id_product_attribute"]);			
				if (empty($cmb_product->id_product))
					return;	// Supprimé entre-temps ?
			}

			// Prix d'achat et prix de vente ?
			if ($this->Link->OPT_ALLOW_I_SALE_PRICES)
			{
				$this->SetPricesContext();
				$pa = isset($data->wp) ? (float) $data->wp : -1;
				$pv = isset($data->sp) ? (float) $data->sp : -1;

				if ($pa == -1 && isset($data["w_price"]))
					$pa = (float) $data["w_price"];
				
				if ($pv == -1 && isset($data["s_price"]))
					$pv = (float) $data["s_price"];
					
				if ($pa >= 0) $pa = checkPrice($pa);
				if ($pv >= 0) $pv = checkPrice($pv);				

				// Changement du prix fournisseur Polaris
				if ($pa >= 0)
				{
					// Soucis avec les PA > PV
					if ($pv != -1 && $pa > $pv)
						$pa = 0;

					$suppliers = new PrestaShopCollection('ProductSupplier');
					$suppliers->sqlWhere("id_product = ".$product["id_product"].
									" AND id_product_attribute = ".$product["id_product_attribute"].
									" AND id_supplier = ".$this->Link->SupplierID);
					foreach ($suppliers as $supplier)
					{
						if ($supplier->product_supplier_price_te != $pa)
						{
							$supplier->product_supplier_price_te = $pa;
							$supplier->save();
						}
					}
				}			
				
				// Changement des prix dans les modèles
				if ($this->Link->OPT_ALLOW_I_SALE_PRICES)
				{
					// Les prix sont spécifiés, il faut les contrôler ...				
					if (!empty($cmb_product))
					{
						$current_pv = checkPrice(floatval($c_product->price + $cmb_product->price), false);

						$pa_changed = $pa >= 0 && ( $cmb_product->wholesale_price != $pa ) && abs($cmb_product->wholesale_price - $pa) >= 0.01;
						$pv_changed = $pv >= 0 && ( $current_pv != $pv ) && abs($current_pv - $pv) >= 0.01;

						if ($pa_changed || $pv_changed)
						{							
							$log = $this->NewLogCtx($cmb_product, "Changement prix du produit %s/%s", [$c_product->reference, $cmb_product->reference]);

							if ($pa_changed)
							{
								$log->LogChange("PAHT", $pa, $cmb_product->wholesale_price);
								$cmb_product->wholesale_price = $pa;
							}
							
							if ($pv_changed)
							{
								$log->LogChange("PVHT", $pv, $current_pv);
								$cmb_product->price = checkPrice($pv - $c_product->price, false);
							}

							$cmb_product->save();
							$log->Commit();
						}
					}
					else
					{
						$current_pv = checkPrice(floatval($c_product->price), false);

						$pa_changed = $pa >= 0 && ( $c_product->wholesale_price != $pa ) && abs($c_product->wholesale_price - $pa) >= 0.01;
						$pv_changed = $pv >= 0 && ( $current_pv != $pv ) && abs($current_pv - $pv) >= 0.01;

						if ($pa_changed || $pv_changed)
						{
							$log = $this->NewLogCtx($cmb_product, "Changement prix du produit %s", [$c_product->reference]);
						
							if ($pa_changed)
							{
								$log->LogChange("PAHT", $pa, $c_product->wholesale_price);
								$c_product->wholesale_price = $pa;								
							}
											
							if ($pv_changed)
							{
								$log->LogChange("PVHT", $pv, $current_pv);
								$c_product->price = $pv;								
							}

							$c_product->save();
							$log->Commit();
						}
					}
				}
			}

			if ($this->Link->OPT_ALLOW_I_STOCKS)
			{
				$this->SetCatalogContext();
				$use_adv_stock = Configuration::get('PS_ADVANCED_STOCK_MANAGEMENT') && $this->Link->OPT_USE_ADV_STOCK;						
				$depends_on_stock = StockAvailable::dependsOnStock($product["id_product"]);

				if ($use_adv_stock)
				{
					// On utilise l'option des stocks avancés, on s'assure que le produit est bien dans 
					// ce mode...				
					if (!$depends_on_stock)
					{
						// On vide le stock manuel et entrepôt déjà fait (on recommence quoi) ...
						Db::getInstance()->update('stock_available', array('quantity' => 0), 'id_product = '.$product["id_product"]);
						Db::getInstance()->update('stock', array('physical_quantity' => 0, 'usable_quantity' => 0), 'id_product = '.$product["id_product"]);
											
						$c_product->setAdvancedStockManagement(1);					
						StockAvailable::setProductDependsOnStock($product["id_product"]);
					}
				}
				else
				{
					// On n'utilise plus l'option des stocks avancés, on s'assure que le produit n'est plus dans
					// ce mode ...
					if ($depends_on_stock)
					{										
						$c_product->setAdvancedStockManagement(0);					
						StockAvailable::setProductDependsOnStock($product["id_product"], 0);
					}
				}
				
				// On cumule ici tous les entrepôts
				$warehouses = array();
				$total_stock = 0;
				foreach ($data->warehouse as $warehouse_data)
				{				
					$qty = (int) $warehouse_data;

					if ($c_product->is_virtual)
						$qty = 10000;

					$total_stock += $qty;
					
					if ($use_adv_stock)
					{
						$code = (string) $warehouse_data["code"];
						if ($code !== "0" && empty($code))
							throw new Exception("l'attribute [code] n'a pas été spécifié alors qu'attendu");

						$warehouse = $this->warehouses->getOrCreateWarehouse($code);					
						if (!isset($warehouses[$warehouse->id]))
							$warehouses[$warehouse->id] = array("wh" => $warehouse, "stock" => 0);
					
						$warehouses[$warehouse->id]["stock"] += $qty;
					}
				}
				
				$done = false;
				if ($use_adv_stock)
				{
					// Stocks avancés					
					foreach ($warehouses as $wh)
					{
						// Cherchons l'ancienne valeur communiquée			
						$warehouse = $wh['wh'];
						$current_stock = $this->stockManager->getProductPhysicalQuantities($product["id_product"], $product["id_product_attribute"], array($warehouse->id));
												
						// Stock à insérer 
						$stock = $wh['stock'];
						$qty = $stock - $current_stock;
						
						if ($qty != 0)
						{
							// On s'assure que le produit est bien associé à l'entrepôt
							if (empty(WarehouseProductLocation::getIdByProductAndWarehouse($product["id_product"], $product["id_product_attribute"], $warehouse->id)))
								Warehouse::setProductLocation($product["id_product"], $product["id_product_attribute"], $warehouse->id, '');
							
							// Chargement du prix d'achat
							if ($pa < 0)
								$pa = empty($cmb_product) ? $c_product->wholesale_price : $cmb_product->wholesale_price;
							
							$done = true;
							if ($qty > 0)
								$this->stockManager->addProduct(
													$product["id_product"],
													$product["id_product_attribute"],
													$warehouse,
													$qty,
													$this->stock_reason,
													$pa
												);
							else
								$this->stockManager->removeProduct(
													$product["id_product"],
													$product["id_product_attribute"],
													$warehouse,
													$qty * -1,
													$this->stock_reason,
													$pa
												);
						}
					}
					
					if ($done)
						StockAvailable::synchronize($product["id_product"]);					
				}
				else
				{
					// On retire le stock minimum de sécurité
					$total_stock -= $this->Link->StockMini;
					if ($total_stock < 0)
						$total_stock = 0;

					// Stocks manuels
					$current_stock = StockAvailable::getQuantityAvailableByProduct(
											$product["id_product"],
											$product["id_product_attribute"]);
					if ($total_stock != $current_stock)
					{
						/*StockAvailable::updateQuantity($product["id_product"], $product["id_product_attribute"], 
							$total_stock - $current_stock);*/
						
						// Set appelle le hook !
						StockAvailable::setQuantity($product["id_product"], $product["id_product_attribute"], $total_stock, null, false);
						$done = true;
					}
				}
				
				if (!empty($this->Link->VisibilityWhenSoldOut))
				{
					// Nous avons changé une pièce en stock, on doit recalculer pour savoir si on tombe sur un stock à 0
					// Et activer / désactiver le produit en conséquence
					if ($c_product->visibility != $this->Link->VisibilityWhenSoldOut && $total_stock <= 0)
					{
						// Le produit était activé, mais il n'y a plus de stock
						// On vérifie le stock du produit entier
						$current_product_stock = StockAvailable::getQuantityAvailableByProduct($c_product->id, 0);
						if ($current_product_stock <= 0)
						{
							// On confirme, plus de stock du tout !
							$this->oneTimeUpdProperty("Visibilité du produit → nulle part (rupture de stock)", $c_product, "visibility", $this->Link->VisibilityWhenSoldOut);
						}
					}
					else if ($c_product->visibility != "both" && $total_stock > 0)
					{
						// Le produit n'était pas activé, mais il y a du stock, donc à forciori, il y a du stock sur tout le produit !
						$this->oneTimeUpdProperty("Visibilité du produit → partout (stock dispo.)", $c_product, "visibility", "both");
					}
				}

				// Stockage table stocks maison (à la fin, comme si erreur, ce n'est pas grave !)
				if (!empty($this->storesMap))
				{					
					// Interrogation des stocks pour éviter des requêtes UPDATE/INSERT sur du stock inchangé					
					$id_product = (int) $product["id_product"];
					$id_product_attribute = (int) $product["id_product_attribute"];

					$stocks = PolarisLinkStockManager::GetMultiStock($id_product);

					foreach ($data->warehouse as $wh)
					{
						$qty = (int) $wh;
						if ($c_product->is_virtual)
							$qty = 10000;

						$key = "code_".$wh['code'];						

						$id_store = null;
						if (isset($this->storesMap[$key]))
							$id_store = (int) $this->storesMap[$key];

						if ($id_store !== null)
						{
							$currentStock = 0;
							if (isset($stocks[$id_product][$id_product_attribute][$id_store]))
								$currentStock = $stocks[$id_product][$id_product_attribute][$id_store];

							if ($currentStock != $qty)
								PolarisLinkStockManager::SetMultiStock($id_store, $id_product, $id_product_attribute, $qty);
						}
					}
				}				
			}
		}
	}
	